@@ -2,6 +2,7 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
+    Coroutine,
     Dict,
     List,
     NewType,
@@ -60,6 +61,14 @@ Wei = NewType('Wei', int)
 Formatters = Dict[RPCEndpoint, Callable[..., Any]]
 
 
+class AccessListEntry(TypedDict):
+    address: HexStr
+    storageKeys: Sequence[HexStr]
+
+
+AccessList = NewType("AccessList", Sequence[AccessListEntry])
+
+
 # todo: move these to eth_typing once web3 is type hinted
 class ABIEventParams(TypedDict, total=False):
     indexed: bool
@@ -127,13 +136,14 @@ class RPCResponse(TypedDict, total=False):
 
 
 Middleware = Callable[[Callable[[RPCEndpoint, Any], RPCResponse], "Web3"], Any]
+AsyncMiddleware = Callable[[RPCEndpoint, Any], Coroutine[Any, Any, RPCResponse]]
 MiddlewareOnion = NamedElementOnion[str, Middleware]
 
 
 class FormattersDict(TypedDict, total=False):
-    error_formatters: Formatters
-    request_formatters: Formatters
-    result_formatters: Formatters
+    error_formatters: Optional[Formatters]
+    request_formatters: Optional[Formatters]
+    result_formatters: Optional[Formatters]
 
 
 class FilterParams(TypedDict, total=False):
@@ -144,6 +154,13 @@ class FilterParams(TypedDict, total=False):
     topics: Sequence[Optional[Union[_Hash32, Sequence[_Hash32]]]]
 
 
+class FeeHistory(TypedDict):
+    baseFeePerGas: List[Wei]
+    gasUsedRatio: List[float]
+    oldestBlock: BlockNumber
+    reward: List[List[Wei]]
+
+
 class LogReceipt(TypedDict):
     address: ChecksumAddress
     blockHash: HexBytes
@@ -160,6 +177,7 @@ class LogReceipt(TypedDict):
 
 # syntax b/c "from" keyword not allowed w/ class construction
 TxData = TypedDict("TxData", {
+    "accessList": AccessList,
     "blockHash": HexBytes,
     "blockNumber": BlockNumber,
     "chainId": int,
@@ -176,6 +194,7 @@ TxData = TypedDict("TxData", {
     "s": HexBytes,
     "to": ChecksumAddress,
     "transactionIndex": int,
+    "type": Union[int, HexStr],
     "v": int,
     "value": Wei,
 }, total=False)
@@ -190,12 +209,13 @@ TxParams = TypedDict("TxParams", {
     "gas": Wei,
     # legacy pricing
     "gasPrice": Wei,
-    # 1559 pricing
+    # dynamic fee pricing
     "maxFeePerGas": Union[str, Wei],
     "maxPriorityFeePerGas": Union[str, Wei],
     "nonce": Nonce,
     # addr or ens
     "to": Union[Address, ChecksumAddress, str],
+    "type": Union[int, HexStr],
     "value": Wei,
 }, total=False)
 
@@ -217,6 +237,7 @@ TxReceipt = TypedDict("TxReceipt", {
     "blockNumber": BlockNumber,
     "contractAddress": Optional[ChecksumAddress],
     "cumulativeGasUsed": int,
+    "effectiveGasPrice": int,
     "gasUsed": Wei,
     "from": ChecksumAddress,
     "logs": List[LogReceipt],
